package immibis.core.api;

import immibis.core.api.net.INetworkingManager;

/**
 * Contains methods to get instances of various API interfaces.
 * 
 * Mods using any APIs should specify "after:ImmibisCore" in their dependency list, so that
 * if the mod includes an outdated API then the newer one will take precedence.
 */
public final class APILocator {
	
	private static <T> T getField(String name) {
		try {
			return (T)Class.forName("immibis.core.ImmibisCore").getField(name).get(null);
		} catch(ClassNotFoundException e) {
			return null;
		} catch(RuntimeException e) {
			throw e;
		} catch(Exception e) {
			throw (AssertionError)new AssertionError("Should not happen").initCause(e);
		}
	}
	
	/**
	 * Returns the ID allocator interface, used to request block and item IDs
	 * and add recipes.
	 * 
	 * Null if Immibis Core is not installed.
	 */
	public static IIDAllocator getIDAllocator() {
		return getField("idAllocator");
	}
	
	/**
	 * Returns the networking manager interface, used to send and listen
	 * for packets.
	 * 
	 * Null if Immibis Core is not installed.
	 */
	public static INetworkingManager getNetManager() {
		return getField("networkingManager");
	}
}
